@@ -29,7 +29,9 @@ TILEDB_KWARG_DEFAULTS = {
     'attrs_filters': None,
     'coords_filters': None,
     'full_domain': False,
-    'tile': None
+    'tile': None,
+    'row_start_idx': None,
+    'fillna': None
 }
 
 def parse_tiledb_kwargs(kwargs):
@@ -53,6 +55,10 @@ def parse_tiledb_kwargs(kwargs):
         args['full_domain'] = kwargs.pop('full_domain')
     if 'tile' in kwargs:
         args['tile'] = kwargs.pop('tile')
+    if 'row_start_idx' in kwargs:
+        args['row_start_idx'] = kwargs.pop('row_start_idx')
+    if 'fillna' in kwargs:
+        args['fillna'] = kwargs.pop('fillna')
 
     return args
 
@@ -130,7 +136,8 @@ def attrs_from_df(df, index_dims=None, filters=None, ctx=None):
 
     return attrs, attr_reprs
 
-def dim_info_for_column(ctx, df, col, tile=None, full_domain=False):
+def dim_info_for_column(ctx, df, col, tile=None, full_domain=False, index_dtype=None):
+
     if isinstance(col, np.ndarray):
         col_values = col
     else:
@@ -139,10 +146,11 @@ def dim_info_for_column(ctx, df, col, tile=None, full_domain=False):
     if len(col_values) < 1:
         raise ValueError("Empty column '{}' cannot be used for dimension!".format(col_name))
 
-    if col_values.dtype is np.dtype('O'):
+    if index_dtype is not None:
+        dim_info = ColumnInfo(index_dtype)
+    elif col_values.dtype is np.dtype('O'):
         col_val0_type = type(col_values[0])
         if col_val0_type in (bytes, unicode_type):
-            dim_min, dim_max = (None, None)
             # TODO... core only supports TILEDB_ASCII right now
             dim_info = ColumnInfo(np.bytes_)
         else:
@@ -152,24 +160,64 @@ def dim_info_for_column(ctx, df, col, tile=None, full_domain=False):
 
     return dim_info
 
-def dim_for_column(ctx, name, dim_info, col, tile=None, full_domain=False):
+def dim_for_column(ctx, name, dim_info, col, tile=None, full_domain=False, ndim=None):
     if isinstance(col, np.ndarray):
         col_values = col
     else:
         col_values = col.values
 
+    if tile is None:
+        if ndim is None:
+            raise TileDBError("Unexpected Nonetype ndim")
+
+        if ndim == 1:
+            tile = 10000
+        elif ndim == 2:
+            tile = 1000
+        elif ndim == 3:
+            tile = 100
+        else:
+            tile = 10
+
+    dtype = dim_info.dtype
+
     if full_domain:
-        # Use the full type domain, deferring to the constructor
-        dim_min, dim_max = (None, None)
+        if not dim_info.dtype in (np.bytes_, np.unicode):
+            # Use the full type domain, deferring to the constructor
+            (dtype_min, dtype_max) = tiledb.libtiledb.dtype_range(dim_info.dtype)
+
+            dim_max = dtype_max
+            if dtype.kind == 'M':
+                date_unit = np.datetime_data(dtype)[0]
+                dim_min = np.datetime64(dtype_min + 1, date_unit)
+                tile_max = np.iinfo(np.uint64).max - tile
+                if np.abs(np.uint64(dtype_max) - np.uint64(dtype_min)) > tile_max:
+                    dim_max = np.datetime64(dtype_max - tile, date_unit)
+            elif dtype is np.int64:
+                dim_min = dtype_min + 1
+            else:
+                dim_min = dtype_min
+
+            if dtype.kind != 'M' and np.issubdtype(dtype, np.integer):
+                tile_max = np.iinfo(np.uint64).max - tile
+                if np.abs(np.uint64(dtype_max) - np.uint64(dtype_min)) > tile_max:
+                    dim_max = dtype_max - tile
+        else:
+            dim_min, dim_max = (None, None)
+
     else:
         dim_min = np.min(col_values)
         dim_max = np.max(col_values)
 
-    if not dim_info.dtype in (np.bytes_, np.unicode) and \
-        np.isscalar(dim_min) and np.isscalar(dim_max):
-        dim_range = np.uint64(np.abs(np.uint64(dim_max) - np.uint64(dim_min)) + 1)
-        if dim_range < tile:
-            tile = dim_range
+    if not dim_info.dtype in (np.bytes_, np.unicode):
+        if np.issubdtype(dtype, np.integer):
+            dim_range = np.uint64(np.abs(np.uint64(dim_max) - np.uint64(dim_min)))
+            if dim_range < tile:
+                tile = dim_range
+        elif np.issubdtype(dtype, np.float64):
+            dim_range = dim_max - dim_min
+            if dim_range < tile:
+                tile = np.ceil(dim_range)
 
     dim = tiledb.Dim(
         name = name,
@@ -193,6 +241,7 @@ def create_dims(ctx, dataframe, index_dims,
     import pandas as pd
     index = dataframe.index
     index_dict = OrderedDict()
+    index_dtype = None
 
     if isinstance(index, pd.MultiIndex):
         for name in index.names:
@@ -202,6 +251,7 @@ def create_dims(ctx, dataframe, index_dims,
         if hasattr(index, 'name') and index.name is not None:
             name = index.name
         else:
+            index_dtype = np.dtype('uint64')
             name = 'rows'
 
         index_dict[name] = index.values
@@ -211,7 +261,8 @@ def create_dims(ctx, dataframe, index_dims,
 
     dim_types = list(
         dim_info_for_column(ctx, dataframe, values,
-                            tile=tile, full_domain=full_domain)
+                            tile=tile, full_domain=full_domain,
+                            index_dtype=index_dtype)
         for values in index_dict.values()
     )
 
@@ -229,21 +280,10 @@ def create_dims(ctx, dataframe, index_dims,
             sparse = True
 
     ndim = len(dim_types)
-    if tile is None:
-        if sparse:
-            tile = 1
-        elif ndim == 1:
-            tile = 10000
-        elif ndim == 2:
-            tile = 1000
-        elif ndim == 3:
-            tile = 100
-        else:
-            tile = 10
 
     dims = list(
         dim_for_column(ctx, name, dim_types[i], values,
-                       tile=tile, full_domain=full_domain)
+                       tile=tile, full_domain=full_domain, ndim=ndim)
         for i, (name, values) in enumerate(index_dict.items())
     )
 
@@ -302,6 +342,8 @@ def from_pandas(uri, dataframe, **kwargs):
     full_domain = args.get('full_domain', False)
     tile = args.get('tile', None)
     nrows = args.get('nrows', None)
+    row_start_idx = args.get('row_start_idx', None)
+    fillna = args.pop('fillna', None)
 
     write = True
     create_array = True
@@ -355,6 +397,10 @@ def from_pandas(uri, dataframe, **kwargs):
 
         tiledb.Array.create(uri, schema, ctx=ctx)
 
+    # apply fill replacements for NA values if specified
+    if fillna is not None:
+        dataframe.fillna(fillna, inplace=True)
+
     if write:
         write_dict = {k: v.values for k,v in dataframe.to_dict(orient='series').items()}
 
@@ -372,7 +418,10 @@ def from_pandas(uri, dataframe, **kwargs):
                 A[tuple(coords)] = write_dict
 
             else:
-                A[:] = write_dict
+                if row_start_idx is None:
+                    row_start_idx = 0
+                row_end_idx = row_start_idx + len(dataframe)
+                A[row_start_idx:row_end_idx] = write_dict
 
             if create_array:
                 write_array_metadata(A, attr_metadata, index_metadata)
@@ -444,9 +493,18 @@ def from_csv(uri, csv_file, **kwargs):
 
     :param uri: URI for new TileDB array
     :param csv_file: input CSV file
-    :param kwargs: optional keyword arguments for Pandas and TileDB.
-                TileDB context and configuration arguments
-                may be passed in a dictionary as `tiledb_args={...}`
+    :param kwargs:
+                - Any pandas.read_csv supported keyword argument.
+                - TileDB-specific arguments:
+                    'allows_duplicates': Generated schema should allow duplicates
+                    'cell_order': Schema cell order
+                    'tile_order': Schema tile order
+                    'mode': (default 'ingest'), Ingestion mode: 'ingest', 'schema_only', 'append'
+                    'full_domain': Dimensions should be created with full range of the dtype
+                    'attrs_filters': FilterList to apply to all Attributes
+                    'coords_filters': FilterList to apply to all coordinates (Dimensions)
+                    'sparse': (default True) Create sparse schema
+                    'tile': Schema tiling (capacity)
     :return: None
 
     **Example:**
@@ -496,7 +554,9 @@ def from_csv(uri, csv_file, **kwargs):
         csv_kwargs = kwargs.copy()
         kwargs.update(tiledb_args)
 
+        rows_written = 0
         for df in pandas.read_csv(csv_file, **csv_kwargs):
+            kwargs['row_start_idx'] = rows_written
             kwargs['full_domain'] = True
             if array_created:
                 kwargs['mode'] = 'append'
@@ -504,6 +564,7 @@ def from_csv(uri, csv_file, **kwargs):
             array_created = True
 
             from_pandas(uri, df, **kwargs)
+            rows_written += len(df)
 
     else:
         df = pandas.read_csv(csv_file, **kwargs)
