@@ -383,8 +383,9 @@ class PandasDataFrameRoundtrip(DiskTestCase):
         import numpy
         ref_schema = tiledb.ArraySchema(
                         domain=tiledb.Domain(*[
-                          tiledb.Dim(name='time', domain=(t0.to_datetime64(), t1.to_datetime64()), tile=1, dtype='datetime64[ns]'),
-                          tiledb.Dim(name='double_range', domain=(-1000.0, 1000.0), tile=1.0, dtype='float64'),
+                          tiledb.Dim(name='time', domain=(t0.to_datetime64(), t1.to_datetime64()),
+                                     tile=1000, dtype='datetime64[ns]'),
+                          tiledb.Dim(name='double_range', domain=(-1000.0, 1000.0), tile=1000, dtype='float64'),
                         ]),
                         attrs=[
                           tiledb.Attr(name='int_vals', dtype='int64', filters=attrs_filters),
@@ -439,6 +440,7 @@ class PandasDataFrameRoundtrip(DiskTestCase):
         df.sort_values('time', inplace=True)
         df.to_csv(tmp_csv, index=False)
 
+        # Test sparse chunked
         tmp_array = os.path.join(tmp_dir, "array")
         tiledb.from_csv(tmp_array, tmp_csv,
                         index_col=['time', 'double_range'],
@@ -450,5 +452,48 @@ class PandasDataFrameRoundtrip(DiskTestCase):
             df_bk = pd.DataFrame(res)
             df_bk.set_index(['time','double_range'], inplace=True)
 
-            df.set_index(['time','double_range'], inplace=True)
+            df_ck = df.set_index(['time','double_range'])
+            tm.assert_frame_equal(df_bk, df_ck)
+
+        # Test dense chunked
+        tmp_array_dense = os.path.join(tmp_dir, "array_dense")
+        tiledb.from_csv(tmp_array_dense, tmp_csv,
+                        parse_dates=['time'],
+                        sparse=False,
+                        chunksize=25)
+
+        with tiledb.open(tmp_array_dense) as A:
+            # chunked writes go to unlimited domain, so we must only read nonempty
+            ned = A.nonempty_domain()[0]
+            # TODO should support numpy scalar here
+            res = A.multi_index[int(ned[0]):int(ned[1])]
+            df_bk = pd.DataFrame(res)
+
+            tm.assert_frame_equal(df_bk, df)
+
+    def test_csv_fillna(self):
+        col_size = 10
+        data = np.random.rand(10)
+        data[4] = np.nan
+        df = pd.DataFrame({'v': data})
+
+        tmp_dir = self.path("csv_fillna")
+        os.mkdir(tmp_dir)
+        tmp_csv = os.path.join(tmp_dir, "generated.csv")
+
+        df.to_csv(tmp_csv, index=False, na_rep="NaN")
+
+        tmp_array = os.path.join(tmp_dir, "array")
+        tiledb.from_csv(tmp_array, tmp_csv, fillna={'v': 0})
+
+        with tiledb.open(tmp_array) as A:
+            res = A[:]
+            # check the value in the raw array
+            self.assertEqual(res['v'][4], 0)
+
+            df_bk = pd.DataFrame(res)
+            df_bk.pop('rows')
+
+            # update the value in the original dataframe to match what we expect on read-back
+            df['v'][4] = 0
             tm.assert_frame_equal(df_bk, df)
\ No newline at end of file
